/*$Id: SenTestSuite.m,v 1.10 2002/01/09 16:46:05 alain Exp $*/

// Copyright (c) 1997-2001, Sen:te (Sente SA).  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenTestSuite.h"
#import "SenTestCase.h"
#import "SenTestSuiteRun.h"
#import <SenFoundation/SenFoundation.h>
#import <Foundation/Foundation.h>

static NSMutableDictionary *suiteForBundleCache = nil;


@interface NSObject (NSBundle)
+ (NSBundle *) bundle;
@end


@implementation NSObject (NSBundle)
+ (NSBundle *) bundle
{
    return [NSBundle bundleForClass:self];
}
@end


@interface SenTestCaseSuite : SenTestSuite
{
    @private
    Class testCaseClass;
}
+ (id) emptyTestSuiteForTestCaseClass:(Class) aClass;
@end


@implementation SenTestSuite

+ (id) testSuiteWithName:(NSString *) aName
{
    return [[[self alloc] initWithName:aName] autorelease];
}


+ (id) emptyTestSuiteNamedFromPath:(NSString *) aPath
{
    return [self testSuiteWithName:[NSString stringWithFormat:@"%@(Tests)", aPath]];
}


+ (void) updateCache
{
    NSEnumerator *testCaseEnumerator = [[SenTestCase allSubclasses] objectEnumerator];
    id each = nil;
    while (each = [testCaseEnumerator nextObject]) {
        NSString *path;

	path = [[each bundle] bundlePath];

        [[suiteForBundleCache objectForKey:path setObjectIfAbsent:[self emptyTestSuiteNamedFromPath:path]] addTest:[each defaultTestSuite]];
    }
}


+ (void) invalidateCache
{
    RELEASE (suiteForBundleCache);
}


+ (NSDictionary *) suiteForBundleCache
{
    if (suiteForBundleCache == nil) {
        suiteForBundleCache = [[NSMutableDictionary alloc] init];
        [self updateCache];
    }
    return suiteForBundleCache;
}



+ (id) testSuiteForBundlePath:(NSString *) bundlePath
{
    if (isNilOrEmpty (bundlePath)) {
        return [self defaultTestSuite];
    }
    else {
        SenTestSuite *suite = [[self suiteForBundleCache] objectForKey:bundlePath];
        return (suite != nil) ? suite : [self emptyTestSuiteNamedFromPath:bundlePath];
    }
}


+ (id) testSuiteForTestCaseWithName:(NSString *) aName
{
    SenTestSuite *suite;
    NSArray *components = [aName pathComponents];
    if ([components count] > 0) {
        Class testCaseClass = NSClassFromString([components senFirstObject]);
        suite = [SenTestCaseSuite emptyTestSuiteForTestCaseClass:testCaseClass];
        if ([components count] == 2) {
            if ([testCaseClass respondsToSelector:@selector (testCaseWithSelector:)]) {
                [suite addTest:[testCaseClass testCaseWithSelector:NSSelectorFromString([components lastObject])]];
            }
        }
        else {
            if ([testCaseClass respondsToSelector:@selector (defaultTestSuite)]) {
                suite = [testCaseClass defaultTestSuite];
            }
        }
        return suite;
    }
    return [self testSuiteWithName:aName];
}


+ (id) testSuiteForTestCaseClass:(Class) aClass
{
    SenTestSuite *suite = [SenTestCaseSuite emptyTestSuiteForTestCaseClass:aClass];

    [suite addTests:
        [aClass 
            collectionByPerformingSelector:@selector(testCaseWithInvocation:)
            withEachObjectInCollection: [aClass testInvocations]]];
    return suite;
}


+ (id <SenCollection>) flatTests
{
    return [[SenTestCase allSubclasses] collectionByPerformingSelector:@selector(defaultTestSuite)];
}


+ (id <SenCollection>) structuredTests
{
    return [self suiteForBundleCache];   
}


+ (id) allTests
{
    SenTestSuite *fullTestSuite = [self testSuiteWithName:@"All tests"];

    [fullTestSuite addTests:[self structuredTests]];
    return fullTestSuite;   
}


+ (id) defaultTestSuite
{
    return [self allTests];
}


- (id) initWithName:(NSString *) aName
{
    self = [super init];
    name = [aName copy];
    tests = [[NSMutableArray alloc] init];
    return self;
}


- (id) initWithCoder:(NSCoder *) aCoder
{
    name = [[aCoder decodeObject] retain];
    tests = [[aCoder decodeObject] retain];
    return self;
}


- (void) encodeWithCoder:(NSCoder *) aCoder
{
    [aCoder encodeObject:name];
    [aCoder encodeObject:tests];
}


- (void) dealloc
{
    RELEASE (name);
    RELEASE (tests);
    [super dealloc];
}


- (NSString *) name
{
    return name;
}


- (NSString *) description
{
    return [self name];
}


- (void) addTest:(SenTest *) aTest
{
    if (aTest != nil) {
        [tests addObject:aTest];   
    }
}


- (void) addTests:(id <SenCollection>) aCollection;
{
    [self performSelector:@selector(addTest:) withEachObjectInCollection:aCollection];
}


- (unsigned int) testCaseCount
{
    return [tests sumWithUnsignedIntSelector:_cmd];
}


- (Class) testRunClass
{
    return [SenTestSuiteRun class];
}


- (void) performTest:(SenTestRun *) aTestRun
{
    [self setUp];
    [aTestRun start];
    [aTestRun performSelector:@selector (addTestRun:) withEachObjectInCollection:[tests collectionByPerformingSelector:@selector(run)]];
    [aTestRun stop];
    [self tearDown];
}
@end


@implementation SenTestCase (SenTestSuiteExtensions)
+ (void) setUp
{
}

+ (void) tearDown
{
}
@end


@implementation SenTestCaseSuite
- (id) initWithTestCaseClass:(Class) aClass
{
    testCaseClass = aClass;
    [super initWithName:NSStringFromClass(testCaseClass)];
    return self;
}


+ (id) emptyTestSuiteForTestCaseClass:(Class) aClass
{
    return [[[self alloc] initWithTestCaseClass:aClass] autorelease];
}


- (void) setUp
{
    [testCaseClass setUp];
}


- (void) tearDown
{
    [testCaseClass tearDown];
}
@end
